<!DOCTYPE html>
<html lang="en">
{%- set reldelim1 = reldelim1 is not defined and ' &raquo;' or reldelim1 %}
{%- set reldelim2 = reldelim2 is not defined and ' |' or reldelim2 %}
{%- set render_sidebar = (not embedded) and (not theme_nosidebar|tobool) and
(sidebars != []) %}
{%- set url_root = pathto('', 1) %}
{# XXX necessary? #}
{%- if url_root == '#' %}{% set url_root = '' %}{% endif %}
{%- if not embedded and docstitle %}
{%- set titlesuffix = " | "|safe + docstitle|e %}
{%- else %}
{%- set titlesuffix = "" %}
{%- endif %}
<head>
    <meta charset="{{ encoding }}">
    <title>{{ title|striptags|e }}{{ titlesuffix }}</title>
    <link rel="stylesheet" href="{{ pathto('_static/pygments.css', 1) }}" type="text/css"/>
    {%- for cssfile in css_files %}
    <link rel="stylesheet" href="{{ pathto(cssfile, 1) }}" type="text/css"/>
    {%- endfor %}
    <link rel="stylesheet" href="{{ pathto('_static/' + style, 1) }}" type="text/css"/>
    {%- if favicon %}
    <link rel="shortcut icon" href="{{ pathto('_static/' + favicon, 1) }}"/>
    {%- endif %}
    {%- block linktags %}
    <link rel="top" title="{{ docstitle|e }}" href="{{ pathto('index') }}"/>
    {%- if parents %}
    <link rel="up" title="{{ parents[-1].title|striptags|e }}" href="{{ parents[-1].link|e }}"/>
    {%- endif %}
    {%- if next %}
    <link rel="next" title="{{ next.title|striptags|e }}" href="{{ next.link|e }}"/>
    {%- endif %}
    {%- if prev %}
    <link rel="prev" title="{{ prev.title|striptags|e }}" href="{{ prev.link|e }}"/>
    {%- endif %}
    {%- endblock %}
    <style lang="text/css">
        #top-bar, #top-bar small, #top-bar a {
            text-shadow: 0px -1px 0px {{theme_gradient_end}};
            color: {{theme_gradient_text}};
        }
        
        #top-bar {
            background-color: {{theme_gradient_bg}};
            background-image: -moz-linear-gradient(top, {{theme_gradient_start}}, {{theme_gradient_end}});
            background-image: -ms-linear-gradient(top, {{theme_gradient_start}}, {{theme_gradient_end}});
            background-image: -webkit-gradient(linear, 0 0, 0 100%, from({{theme_gradient_start}}), to({{theme_gradient_end}}));
            background-image: -webkit-linear-gradient(top, {{theme_gradient_start}}, {{theme_gradient_end}});
            background-image: -o-linear-gradient(top, {{theme_gradient_start}}, {{theme_gradient_end}});
            background-image: linear-gradient(top, {{theme_gradient_start}}, {{theme_gradient_end}});
            filter: progid:DXImageTransform.Microsoft.gradient(startColorstr = '{{theme_gradient_start}}', endColorstr = '{{theme_gradient_end}}', GradientType = 0);
            -webkit-border-radius: 0;
            -moz-border-radius: 0;
            border-radius: 0;
        }


        .hero-unit {
            background-image: url("{{ pathto('_static/' + theme_landing_logo, 1) }}") !important;
            background-repeat: no-repeat !important;
            background-position: 30px 50px;
        }

        .hero-unit div.section {
            padding-left: {{theme_landing_logo_width}} !important;
        }
    </style>
</head>
<body>
    <a href="{{theme_github_page}}">
        <img style="position: absolute; top: 0; right: 0; border: 0;"
             src="https://s3.amazonaws.com/github/ribbons/forkme_right_orange_ff7600.png"
             alt="Fork me on GitHub"></a>
    <div class="navbar">
        <div class="navbar-inner container-fluid" id="top-bar">
            <header class="row-fluid">
                <h1 class="span12" id="title">
                    {%- block sidebarlogo %}
                    {%- if logo %}<img class="logo" src="{{ pathto('_static/' + logo, 1) }}" alt="Logo"/>{%- endif %}
                    {%- endblock %}
                    <a href="{{ pathto(master_doc) }}">{{ docstitle|striptags|e }}</a>
                    <small>{{theme_tagline|striptags|e}}</small>
                </h1>
            </header>
        </div>
    </div>
    <div class="container-fluid">
        <div class="row-fluid">
            {%- block sidebar %}
            {%- if title != "Home" %}
            <div class="span3" id="sidebar">
                {{ toctree(maxdepth=-1) }}
                <hr/>
                <ul>
                    <li><a href="{{theme_apidocs}}">Javadoc</a></li>
                    <li><a href="{{theme_mailing_list}}">Mailing List</a></li>
                </ul>
            </div>
            {%- endif %}
            {%- endblock %}
            <div class="{%- if title == "Home" %}span12{%- else %}span9{%- endif %}" id="body">
                {% block body %} {% endblock %}
            </div>
        {%- if title == "Home" -%}
        <div>
        YourKit is kindly supporting the Metrics project with its full-featured Java Profiler.
        YourKit, LLC
        is the creator of innovative and intelligent tools for profiling Java and .NET
        applications. Take a look at YourKit's leading software products:
        <a href="http://www.yourkit.com/java/profiler/index.jsp">YourKit Java Profiler</a> and
        <a href="http://www.yourkit.com/.net/profiler/index.jsp">YourKit .NET Profiler</a>.
        </div>
        {%- endif -%}
        </div>
        <hr/>
        <footer>
            <p style="float: left">
            {%- if show_copyright %}
            {%- if hasdoc('copyright') %}
            {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a href="{{ path }}">Copyright</a>
            {{ copyright }}.{% endtrans %}
            {%- else %}
            {% trans copyright=copyright|e %}&copy; Copyright {{ copyright }}.{% endtrans %}
            {%- endif %}
            {%- endif %}
            {%- if last_updated %}
            {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
            {%- endif %}
            {%- if show_sphinx %}
            {% trans sphinx_version=sphinx_version|e %}Created using <a href="http://sphinx.pocoo.org/">Sphinx</a>
            {{ sphinx_version }}.{% endtrans %}
            {%- endif %}
            </p>
            <p style="float: right">Dropwizard Metrics v{{ release }}</p>
        </footer>
    </div>
</body>
</html>



