@import "reset.less";
@import "variables.less"; // Modify this for custom colors, font-sizes, etc
@import "mixins.less";
@import "scaffolding.less";
@import "grid.less";
@import "layouts.less";
@import "type.less";
@import "code.less";
@import "tables.less";
@import "buttons.less";
@import "navs.less";
@import "navbar.less";
@import "hero-unit.less";
@import "utilities.less"; // Has to be last to override when necessary

#call-to-action {
    text-align: right;
}

a.headerlink {
    display: none;
}

#title {
    color: #ffffff;
}

.hero-unit h1 {
    padding-bottom: 20px ! important;
}

#top-bar small {
    color: #f8f8ff;
    text-shadow: 0px -1px 0px #5f0c17;
}

.admonition {
    padding: 14px 35px 14px 14px;
    margin-bottom: 18px;
    text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
    background-color: #fcf8e3;
    border: 1px solid #fbeed5;
    -webkit-border-radius: 4px;
    -moz-border-radius: 4px;
    border-radius: 4px;
}

.admonition .admonition-title {
    font-size: 14pt;
    font-weight: bold;
}

.admonition.note .admonition-title,
.admonition-todo .admonition-title {
    color: #c09853;
}

.admonition.tip,
.admonition.hint {
    background-color: #dff0d8;
    border-color: #d6e9c6;
}

.admonition.tip .admonition-title,
.admonition.hint .admonition-title {
    color: #468847;
}

.admonition.error,
.admonition.warning,
.admonition.caution,
.admonition.danger,
.admonition.attention {
    background-color: #f2dede;
    border-color: #eed3d7;
}

.admonition.error .admonition-title,
.admonition.warning .admonition-title,
.admonition.caution .admonition-title,
.admonition.danger .admonition-title,
.admonition.attention .admonition-title {
    color: #b94a48;
}

.admonition.important {
    background-color: #d9edf7;
    border-color: #bce8f1;
}

.admonition.important .admonition-title {
    color: #3a87ad;
}

.admonition > p, .admonition > ul {
    margin-bottom: 0;
}

.admonition p + p {
    margin-top: 5px;
}

a.internal.reference > em {
    font-style: normal ! important;
    text-decoration: none ! important;
}

tt {
    .code();
}

.section > p, .section ul li, .admonition p, .section dt, .section dl {
    font-size: 13pt;
    line-height: 18pt;
}

.section tt {
    font-size: 11pt;
    line-height: 11pt;
}

.section > * {
    margin-bottom: 20px;
}

pre {
    font-family: 'Panic Sans', Menlo, Monaco, Consolas, Andale Mono, Courier New, monospace !important;
    font-size: 12pt !important;
    line-height: 22px !important;
    display: block !important;
    width: auto !important;
    height: auto !important;
    overflow: auto !important;
    white-space: pre !important;
    word-wrap: normal !important;
}

#body h1, h1 tt {
    font-size: 28pt;
}

h1 tt {
    background-color: transparent;
    font-size: 26pt !important;
}

#body h2 {
    font-size: 24pt;
}

h2 tt {
    background-color: transparent;
    font-size: 22pt !important;
}

#body h3 {
    font-size: 20pt;
}

h3 tt {
    background-color: transparent;
    font-size: 18pt !important;
}

#body h4 {
    font-size: 16pt;
}

h4 tt {
    background-color: transparent;
    font-size: 14pt !important;
}

#sidebar tt {
    color: #08c;
    background-color: transparent;
}

.hero-unit .toctree-wrapper {
    text-align: center;
}

.hero-unit li {
    display: inline;
    list-style-type: none;
    padding-right: 20px;
}

.hero-unit li a {
    .btn();
    .btn-success();
    padding:10px 10px 10px;
    font-size:16pt;
    &:hover {
        color: @grayDark;
        text-decoration: none;
        background-color: darken(@white, 10%);
        background-position: 0 -15px;

        // transition is only when going to hover, otherwise the background
        // behind the gradient (there for IE<=9 fallback) gets mismatched
        .transition(background-position .1s linear);
        .btn-success();
    }

    &:focus {
        .tab-focus();
        .btn-success();
    }

    &:active {
        background-image: none;
        @shadow: inset 0 2px 4px rgba(0,0,0,.15), 0 1px 2px rgba(0,0,0,.05);
        .box-shadow(@shadow);
        background-color: darken(@white, 10%);
        background-color: darken(@white, 15%) e("\9");
        color: rgba(0,0,0,.5);
        outline: 0;
        .btn-success();
    }
}

.hero-unit li a:after {
    content: " »";
}

table.docutils {
    border: 1px solid #DDD;
    .table();
    .table-striped();
}
