// Patterns.less
// Repeatable UI elements outside the base styles provided from the scaffolding
// ----------------------------------------------------------------------------


// PAGE HEADERS
// ------------

footer {
  padding-top: @baseLineHeight - 1;
  margin-top: @baseLineHeight - 1;
  border-top: 1px solid #eee;
}
