/*!
 * Bootstrap @VERSION for Print
 *
 * Copyright 2012 Twitter, Inc
 * Licensed under the Apache License v2.0
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Designed and built with all the love in the world @twitter by @mdo and @fat.
 * Date: @DATE
 */


// HIDE UNECESSARY COMPONENTS
// --------------------------

.navbar-fixed {
  display: none;
}